@@ -1,9 +1,11 @@
 """ Manges a device under test which can be simulated by Hdev using the TCAD Interface class.
 
-Author: Markus Müller 
+Author: Markus Müller
 """
 # DMT_core
-# Copyright (C) 2019  Markus Müller and Mario Krattenmacher and the DMT contributors <https://gitlab.hrz.tu-chemnitz.de/CEDIC_Bipolar/DMT/>
+# Copyright (C) from 2022  SemiMod
+# Copyright (C) until 2021  Markus Müller, Mario Krattenmacher and Pascal Kuthe
+# <https://gitlab.com/dmt-development/dmt-core>
 #
 # This file is part of DMT_core.
 #
@@ -19,6 +21,8 @@ Author: Markus Müller
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>
+
+# pylint: disable=no-member
 import re
 import copy
 import os
@@ -26,6 +30,7 @@ import numpy as np
 import pandas as pd
 import logging
 import h5py
+from typing import cast
 
 from DMT.core import (
     DutTcad,
@@ -60,7 +65,7 @@ class DutHdev(DutTcad):
     r"""Manages simulations and simulation data with Hdev.
 
     Parameters
-    -----------
+    ----------
     database_dir : str
         Directory for the current databases
     dut_type   : :class:`~DMT.core.dut_type.DutType`
@@ -78,7 +83,7 @@ class DutHdev(DutTcad):
         Arguments for the system command
 
     Attributes
-    -----------
+    ----------
     sim_name : str
         DEVICE simulation title. Sort of a DUT-Name.
 
@@ -111,7 +116,7 @@ class DutHdev(DutTcad):
         inp_ : dict of dicts that describes a Hdev input structure
 
         Returns
-        --------
+        -------
         inp_header : str
         """
         if not "BIAS_DEF" in inp_:
@@ -132,6 +137,7 @@ class DutHdev(DutTcad):
         sweep = copy.deepcopy(sweep)
         """Adds bias blocks to a Hdev heade file (string)"""
         inp_str = self._inp_header
+        inp_str = cast(str, inp_str)
         # find frequency sweep def
         has_f_def = False
         for i, sub_sweep in enumerate(sweep.sweepdef):
@@ -139,7 +145,7 @@ class DutHdev(DutTcad):
                 has_f_def = True
                 break
         if has_f_def:
-            f_def = sweep.sweepdef.pop(i)
+            f_def = sweep.sweepdef.pop(i)  # type: ignore
         sweep.set_values()
         df = sweep.create_df()
         # add T sweep if not specified
@@ -249,7 +255,7 @@ class DutHdev(DutTcad):
             Sweep that has been simulated.
 
         Raises
-        --------
+        ------
         NotImplementedError
             If the Dut is not a simulatable dut.
         SimulationUnsuccessful
@@ -336,14 +342,14 @@ class DutHdev(DutTcad):
             Sweep that has been simulated for the desired output files.
 
         Returns
-        ----------
+        -------
         output_files : [string]
             List of strings that contain the full path to the output files.
         delete_sim_results : {False, True}, optional
             If True, the simulation folder is deleted after reading.
 
         Notes
-        ------
+        -----
         .. todo:: Add more column renaming and resorting here!
         """
         # list_elpa_files = re.findall(
@@ -738,14 +744,14 @@ class DutHdev(DutTcad):
                 "cont_name": cont_name,
                 "bias_fun": bias_fun,
             }
-        elif sub_sweep.sweep_type[0:3] == "CON":
-            bias_fun = "'LIN'"  # we convert to list, to support list=1 setting used by DMT in Hdev
+        elif sub_sweep.sweep_type.startswith("CON"):
+            bias_fun = "'TAB'"  # we convert to list, to support list=1 setting used by DMT in Hdev
             bias_info = {
                 "cont_name": cont_name,
                 "bias_fun": bias_fun,
-                "bias_val": [sub_sweep.value_def[0], sub_sweep.value_def[0], len(bias_val)],
+                "bias_val": [sub_sweep.value_def[0]],
             }
-        elif sub_sweep.sweep_type[0:4] == "SYNC":
+        elif sub_sweep.sweep_type.startswith("SYNC"):
             # find master sweep
             for other_sub_sweep in sweep.sweepdef:
                 if other_sub_sweep.var_name == sub_sweep.master_var:
@@ -759,13 +765,13 @@ class DutHdev(DutTcad):
                     dummy = 1
                     break
 
-        elif sub_sweep.sweep_type[0:4] == "LIST":
+        elif sub_sweep.sweep_type.startswith("LIST"):
             bias_fun = "'TAB'"  # we convert to list, to support list=1 setting used by DMT in Hdev
             bias_info = {"cont_name": cont_name, "bias_fun": bias_fun, "bias_val": sub_sweep.values}
         else:
             raise NotImplementedError
 
-        return bias_info
+        return bias_info  # type: ignore
 
     def get_mobility(self, semiconductor, valley, field, temperature, doping, grading):
         if not DutHdev.inited:
@@ -880,10 +886,12 @@ def getITRSHBT(
     xj_l=None,
     mu_min_a=None,
     mu_min_d=None,
+    beta=None,
 ):
     """Return a DutHdev for ITRS HBTS. Only for SiGe HBTs.
-    Input
-    -----
+
+    Parameters
+    ----------
     hbt_fun : callable
         Function that returns a Hdev HBT definition.
     fermi : bool
@@ -896,7 +904,7 @@ def getITRSHBT(
         If True: use ET model, else DD model.
 
     Returns
-    --------
+    -------
     dut_hdev : DutHdev
         A DMT.Hdev.DutHdev object ready for simulation.
     """
@@ -927,6 +935,10 @@ def getITRSHBT(
     if not sat:
         for i, _mob in enumerate(inp["MOB_DEF"]):
             inp["MOB_DEF"][i]["hc_scat_type"] = "default"
+    else:
+        for i, _mob in enumerate(inp["MOB_DEF"]):
+            if inp["MOB_DEF"][i]["valley"] == "x":
+                inp["MOB_DEF"][i]["hc_scat_type"] = "sat"
 
     if mu_min_a is not None:
         # decrease lattice mobility
@@ -939,9 +951,12 @@ def getITRSHBT(
         for l, mob in enumerate(inp["MOB_DEF"]):
             if inp["MOB_DEF"][l]["valley"] == "X":
                 inp["MOB_DEF"][l]["mu_min_d"] = inp["MOB_DEF"][l]["mu_min_d"] * mu_min_d
-    # if te:
-    #     inp["THERMIONIC_EMISSION"] = {}
-    #     inp["THERMIONIC_EMISSION"]["x"] = 32e-9
+
+    if beta is not None:
+        # decrease lattice mobility
+        for l, mob in enumerate(inp["MOB_DEF"]):
+            if inp["MOB_DEF"][l]["valley"] == "X":
+                inp["MOB_DEF"][l]["beta"] = 1.5
 
     inp = turn_off_recombination(inp)
 
@@ -950,7 +965,4 @@ def getITRSHBT(
         DutType.npn,
         inp,
         reference_node="E",
-        list_copy=os.path.join(
-            "/home/markus/Documents/Gitprojects/dissertation/py_scripts/tsc250_tcad/teledyne_profile.h5"
-        ),
     )
