@@ -1,5 +1,7 @@
 # DMT_core
-# Copyright (C) 2019  Markus Müller and Mario Krattenmacher and the DMT contributors <https://gitlab.hrz.tu-chemnitz.de/CEDIC_Bipolar/DMT/>
+# Copyright (C) from 2022  SemiMod
+# Copyright (C) until 2021  Markus Müller, Mario Krattenmacher and Pascal Kuthe
+# <https://gitlab.com/dmt-development/dmt-core>
 #
 # This file is part of DMT_core.
 #
@@ -16,35 +18,20 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>
 
-# dirty: check which modules available
+from pint import UnitRegistry
+from pathlib import Path
 
-from importlib import util
+try:
+    from semver.version import Version as VersionInfo
+except ImportError:
+    from semver import VersionInfo
 
-core_exists = util.find_spec("DMT.core") is not None
-extraction_exists = util.find_spec("DMT.extraction") is not None
-
-# if core_exists and not extraction_exists:
-#     mode = "free version"
-# elif core_exists and extraction_exists:
-#     mode = "all"
-
-print("-----------------------------------------------------------------------")
-print("DMT Copyright (C) 2020 Markus Müller, Mario Krattenmacher, Pascal Kuthe")
-print("This program comes with ABSOLUTELY NO WARRANTY.")
-print("DMT_core is free software, and you are welcome to redistribute it.")
-print("DMT_other is free for non-commercial use, see LICENSE.md. ")
-print("-----------------------------------------------------------------------")
+__version__ = VersionInfo(major=1, minor=6, patch=1)  # , prerelease="rc.1"
+# to get the next version:
+# __version__.next_version(x) - with x = "major", "minor", "patch", "prerelease"
 
 name = "core"
 
-import numpy as np
-
-# compile cython DMT modules
-import pyximport
-
-pyximport.install(setup_args={"include_dirs": np.get_include()}, language_level=3)
-
-import os
 
 from . import constants
 
@@ -53,25 +40,17 @@ from .singleton import Singleton
 from .hasher import create_md5_hash
 
 # one unit registry for all of DMT
-from pint import UnitRegistry
 
 unit_registry = UnitRegistry()
 from DMT.config import DATA_CONFIG
 
-from pkg_resources import Requirement, resource_filename
 
-path_core = os.path.dirname(os.path.abspath(__file__))
-unit_registry.load_definitions(os.path.join(path_core, "dmt_units.txt"))
+path_core = Path(__file__).resolve().parent
+unit_registry.load_definitions(str(path_core / "dmt_units.txt"))
 
 # helper for model equations and mcard memoization
 from .utils import print_progress_bar
-from .utils import check_nan_inf
-from .utils import vectorize
-from .utils import flatten
-from .utils import strictly_increasing
 from .utils import enumerate_reversed
-from .utils import resolve_siunitx
-from .utils import tex_to_text
 
 # naming conventions
 from .naming import specifiers
@@ -85,49 +64,39 @@ from .naming import get_sub_specifiers
 from .naming import natural_scales
 
 # compact modelling stuff
-from .mc_parameter import McParameter, McParameterComposition
-from .utils import memoize
+from .mc_parameter import McParameter, McParameterCollection, McParameterComposition
+from .va_file import VAFile
 from .mcard import MCard
 from .technology import Technology
+from .circuit import Circuit, CircuitElement
 
 # plotting
 from .plot import Plot
-from .plot import SmithPlot
 from .plot import save_or_show
-from .plot import Plot2YAxis
 from .plot import COMPARISON_3
-
-from .tikz_postprocess import TikzPostprocess
+from .plot_smith import SmithPlot
+from .plot_2yaxis import Plot2YAxis
 
 # Data management and processing
-try:
-    from .data_processor_pyx import DataProcessor
-except ImportError:
-    from .data_processor_py import DataProcessor
+from .data_processor import is_iterable, flatten, strictly_increasing, DataProcessor
 
 from .data_frame import DataFrame
-from .sweep import Sweep, SweepDef
+from .sweep import Sweep, SweepDef, get_sweepdef
 from .database_manager import DatabaseManager
+from .data_reader import (
+    read_data,
+    save_elpa,
+    read_ADS_bin,
+    read_DEVICE_bin,
+    read_elpa,
+    read_mdm,
+    read_hdf,
+    read_feather,
+)
 
-# try:
-#     from .data_reader_pyx     import read_data, read_ADS_bin, read_DEVICE_bin, read_elpa, read_mdm, read_hdf
-# except ImportError:
-if True:
-    from .data_reader_py import (
-        read_data,
-        read_ADS_bin,
-        read_DEVICE_bin,
-        read_elpa,
-        read_mdm,
-        read_hdf,
-        read_feather,
-    )
-
-from .circuit import Circuit, CircuitElement
 
 # Simulation management
 from .sim_con import SimCon
-from .df_to_sweep import df_to_sweep
 
 # DutView tree
 from .dut_view import DutView
@@ -141,4 +110,28 @@ from .dut_circuit import DutCircuit
 
 from .dut_tcad import DutTcad
 
-import tables
+# docu
+from .docu_dut_lib import DocuDutLib
+
+# determine which modules are present
+core_exists = True  # always, without DMT is not possible
+try:
+    import DMT.extraction
+
+    extraction_exists = True
+except ImportError:
+    extraction_exists = False
+
+
+if core_exists and not extraction_exists:
+    mode = "free version"
+elif core_exists and extraction_exists:
+    mode = "all"
+
+print("-----------------------------------------------------------------------")
+print("DMT Copyright (C) 2022 SemiMod")
+print("This program comes with ABSOLUTELY NO WARRANTY.")
+print("DMT_core is free software, and you are welcome to redistribute it.")
+if extraction_exists:
+    print("DMT_other is free for non-commercial use, see LICENSE.md. ")
+print("-----------------------------------------------------------------------")
