@@ -2,7 +2,9 @@
 
 """
 # DMT_core
-# Copyright (C) 2019  Markus Müller and Mario Krattenmacher and the DMT contributors <https://gitlab.hrz.tu-chemnitz.de/CEDIC_Bipolar/DMT/>
+# Copyright (C) from 2022  SemiMod
+# Copyright (C) until 2021  Markus Müller, Mario Krattenmacher and Pascal Kuthe
+# <https://gitlab.com/dmt-development/dmt-core>
 #
 # This file is part of DMT_core.
 #
@@ -29,7 +31,7 @@ class DutMeas(DutView):
     """Subclass of DutView that is used to capture relevant methods and properties of measured data.
 
     Parameters
-    -----------
+    ----------
     database_dir         :  string
         Root directory for saving duts and databases of this project.
     name           :  string
@@ -49,7 +51,7 @@ class DutMeas(DutView):
         Unique hash of this object.
 
     Attributes
-    -----------
+    ----------
     width          :  float64
         Width in m.
     length         :  float64
@@ -117,7 +119,7 @@ class DutMeas(DutView):
         """Raise a OSError. Measurements can not be simulated!
 
         Raises
-        -------
+        ------
         OSError
             Always when called.
         """
@@ -127,7 +129,7 @@ class DutMeas(DutView):
         """Raise a OSError. Measurement duts can not be simulated!
 
         Raises
-        -------
+        ------
         OSError
             Always when called.
         """
@@ -137,7 +139,7 @@ class DutMeas(DutView):
         """Should genrate the input file for a given simulation sweep. This is not possible for measurements!
 
         Raises
-        -------
+        ------
         OSError
             Always when called.
         """
@@ -147,7 +149,7 @@ class DutMeas(DutView):
         """Empty string, so it is evaluated to false, but still a string.
 
         Returns
-        --------
+        -------
         ''
         """
         return ""
@@ -273,58 +275,14 @@ class DutMeas(DutView):
             Function to convert a key part into a temperature.
 
         Returns
-        --------
+        -------
         key : str
             The correctly joined key_parts. If the temperature was inside, it is converted into "T" + temp + "K".
 
         """
 
-        def temp_converter_default(key_part):
-            """Default converter from key_part to temperature. Asumes the temperature in the key parts is in Kelvin.
-
-            Parameters
-            ----------
-            key_part : str
-                Key to inspect
-
-            Returns
-            --------
-            temp : float or None
-                If key does not contain a temperature, -1 is returned.
-                If id does contain a temperature, the temperature in Kelvin is returned.
-            """
-            temp = None
-            re_temp = re.search(r"T([0-9p\.]+)K", key_part)
-            if re_temp:
-                try:
-                    # always replace "p" with ".", if it is already with ".", it doesn't matter
-                    temp = round(float(re_temp.group(1).replace("p", ".")), 3)
-                except ValueError:
-                    # if a value error in the except clause happens, try the next key part.
-                    pass
-
-            # alternative:
-            if key_part.startswith("T"):
-                try:
-                    temp = round(float(key_part[1:].replace("p", ".")), 3)
-                except ValueError:
-                    # if a value error in the except clause happens, try the next key part.
-                    pass
-
-            # finally as a last escape: direct conversion :(
-            try:
-                temp = round(float(key_part.replace("p", ".")), 3)
-            except ValueError:
-                # if a value error in the except clause happens, try the next key part.
-                pass
-
-            if temp is None:
-                return -1
-            else:
-                return temp
-
         if temperature_converter is None:
-            temperature_converter = temp_converter_default
+            temperature_converter = self.temp_converter_default
 
         key_parts = list(key_parts)
         for i_key_part, key_part in enumerate(key_parts):
@@ -349,3 +307,47 @@ class DutMeas(DutView):
             key_parts[i_key_part] = "T{:.2f}K".format(temp)
 
         return self.join_key(*key_parts)
+
+    def temp_converter_default(self, key_part):
+        """Default converter from key_part to temperature. Asumes the temperature in the key parts is in Kelvin.
+
+        Parameters
+        ----------
+        key_part : str
+            Key to inspect
+
+        Returns
+        -------
+        temp : float or None
+            If key does not contain a temperature, -1 is returned.
+            If id does contain a temperature, the temperature in Kelvin is returned.
+        """
+        temp = None
+        re_temp = re.search(r"T([0-9p\.]+)K", key_part)
+        if re_temp:
+            try:
+                # always replace "p" with ".", if it is already with ".", it doesn't matter
+                temp = round(float(re_temp.group(1).replace("p", ".")), 3)
+            except ValueError:
+                # if a value error in the except clause happens, try the next key part.
+                pass
+
+        # alternative:
+        if key_part.startswith("T"):
+            try:
+                temp = round(float(key_part[1:].replace("p", ".")), 3)
+            except ValueError:
+                # if a value error in the except clause happens, try the next key part.
+                pass
+
+        # finally as a last escape: direct conversion :(
+        try:
+            temp = round(float(key_part.replace("p", ".")), 3)
+        except ValueError:
+            # if a value error in the except clause happens, try the next key part.
+            pass
+
+        if temp is None:
+            return -1
+        else:
+            return temp
