@@ -3,7 +3,9 @@ r""" Global namings for DMT
 Internally all variables and column names which contain a quantity of a specifier must have the same given names, e.g. all voltages will be called: 'V\_'
 """
 # DMT_core
-# Copyright (C) 2019  Markus Müller and Mario Krattenmacher and the DMT contributors <https://gitlab.hrz.tu-chemnitz.de/CEDIC_Bipolar/DMT/>
+# Copyright (C) from 2022  SemiMod
+# Copyright (C) until 2021  Markus Müller, Mario Krattenmacher and Pascal Kuthe
+# <https://gitlab.com/dmt-development/dmt-core>
 #
 # This file is part of DMT_core.
 #
@@ -20,8 +22,10 @@ Internally all variables and column names which contain a quantity of a specifie
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>
 
+from __future__ import annotations
 import copy
 import numpy as np
+from typing import List, Union
 from pint.formatting import siunitx_format_unit
 from more_itertools import unique_everseen
 from DMT.config import DATA_CONFIG
@@ -52,6 +56,9 @@ UNIT_PREFIX_MIX = {
         1e-5: r"\kilo\volt\per\centi\meter",
     },  # field
 }
+UNIT_PREFIX_DENOMINATOR = {
+    1e-6: r"\centi",
+}
 
 
 class SpecifierStr(str):
@@ -62,29 +69,38 @@ class SpecifierStr(str):
     Yeah it is already implemented :P
 
     Parameters
-    -----------
+    ----------
     specifier : str
     nodes : [str], optional
     sub_specifiers : [str], str, {'', sub_specifier}, optional
 
     Attributes
-    ------------
+    ----------
     specifier : str
     nodes : [str]
     sub_specifiers : [str]
 
     """
 
+    specifier: str = ""
+    nodes: List[str] = []
+    sub_specifiers: List[str] = []
+
     # pylint: disable=no-member
-    def __new__(cls, specifier, *nodes, sub_specifiers=None):
+    def __new__(
+        cls,
+        specifier: Union[str, SpecifierStr],
+        *nodes: str,
+        sub_specifiers: Union[List[Union[str, SpecifierStr]], str, SpecifierStr, None] = None,
+    ):
         if not nodes and sub_specifiers is None:
             try:
-                nodes = specifier.nodes
-                sub_specifiers = specifier.sub_specifiers
-                specifier = specifier.specifier
+                nodes = specifier.nodes  # type: ignore
+                sub_specifiers = specifier.sub_specifiers  # type: ignore
+                specifier = specifier.specifier  # type: ignore
             except AttributeError:
                 # revert old state if an Attribute error occurs!
-                nodes = []
+                nodes = ()
                 sub_specifiers = None
 
         # cast to list
@@ -112,12 +128,13 @@ class SpecifierStr(str):
         #     # Not done because parameter sub_specifiers replaces the module attribute
         #     raise SpecifierNotKnown('The sub_specifiers part of a column name can only be one of the following:\n' + str(SUB_SPECIFIERS_STR))
 
-        if nodes and "".join(nodes):
-            nodes = [str(node) for node in nodes]  # string cast :/
+        if nodes and "".join(nodes):  # string cast :/
+            nodes = [str(node) for node in nodes]  # type: ignore
             node_str = "_" + "".join(nodes)
         else:
             node_str = ""
-            nodes = []  # make sure nodes is a list not a tuple
+            nodes = []  # type: ignore
+            # make sure nodes is a list not a tuple
 
         if sub_specifiers:
             sub_specifiers_str = "|" + "|".join(sub_specifiers)
@@ -128,11 +145,11 @@ class SpecifierStr(str):
 
         obj = str.__new__(cls, col_name)
         obj.specifier = str(specifier)
-        obj.nodes = nodes
-        obj.sub_specifiers = sub_specifiers
+        obj.nodes = nodes  # type: ignore
+        obj.sub_specifiers = sub_specifiers  # type: ignore
         return obj
 
-    def get_tex_unit(self, scale=1, add=""):
+    def get_tex_unit(self, scale=1, add="") -> str:
         r"""Get the unit of a given specifier in base units.
 
         If scale is different from one, a suitable unit prefix is chosen. E.g. scale=1e3 and specifier=CURRENT -> \si{\milli\ampere}
@@ -145,7 +162,7 @@ class SpecifierStr(str):
             Additional string that is added to the unit
 
         Returns
-        ------
+        -------
         unit : string
             TeX representation of the specifer's unit
         """
@@ -165,17 +182,24 @@ class SpecifierStr(str):
 
         else:  # pure unit
             # for non-mixed quantities like voltages and current
-            unit_prefix = UNIT_PREFIX[scale]
+
             # get unit from pint
             try:
-                unit = siunitx_format_unit(unit)
+                unit = siunitx_format_unit(unit)  # type: ignore
             except TypeError:
                 unit = siunitx_format_unit(
                     unit._units, unit_registry
                 )  # new version has other interface
-            return r"\si{" + unit_prefix + unit + add + r"}"
 
-    def to_label(self, scale=1, negative=False, divide_by_unit=False):
+            if unit.startswith("\\per"):
+                # for 1/m^3 -> unit prefix should be in the denominator
+                unit = "\\per" + UNIT_PREFIX_DENOMINATOR[scale] + unit[4:]
+            else:
+                unit = UNIT_PREFIX[scale] + unit
+
+            return r"\si{" + unit + add + r"}"
+
+    def to_label(self, scale=1, negative=False, divide_by_unit=False) -> str:
         """Generates a label for plots for this specifier, where scale determines the unit prefix.
 
         Parameters
@@ -215,7 +239,7 @@ class SpecifierStr(str):
             else:
                 return r"$" + self.to_tex() + left_brack + tex_unit + right_brack + r"$"
 
-    def to_legend_with_value(self, value, scale=1, **kwargs):
+    def to_legend_with_value(self, value, scale=1, **kwargs) -> str:
         """Creates a SI legend entry in the form : specifier_tex = \\SI{value}{scale, spec_unit}
 
         Parameters
@@ -232,9 +256,11 @@ class SpecifierStr(str):
         str
             math tex legend entry
         """
+        value = np.real(value)
+
         unit = self.get_pint_unit()
         try:
-            unit = siunitx_format_unit(unit)
+            unit = siunitx_format_unit(unit)  # type: ignore
         except TypeError:
             unit = siunitx_format_unit(
                 unit._units, unit_registry
@@ -250,7 +276,7 @@ class SpecifierStr(str):
 
             return f"${self.to_tex(**kwargs):s}=\\SI{{{value * scale:.2f}}}{{{unit_prefix:s}{unit:s}}}$"
 
-    def to_raw_string(self):
+    def to_raw_string(self) -> str:
         """get a raw string from the specifier -> can be used for string operations and variable naming...
 
         Returns
@@ -260,7 +286,7 @@ class SpecifierStr(str):
         """
         return str(self)  # not tested
 
-    def string_to_save(self):
+    def string_to_save(self) -> str:
         """Generates a single string which allows to identify the specifers, nodes and subspecifiers. Used to save into HDF5 as there the columns have to be valid strings."""
         return (
             "$"
@@ -272,7 +298,7 @@ class SpecifierStr(str):
         )
 
     @classmethod
-    def string_from_load(cls, string):
+    def string_from_load(cls, string) -> str:
         """Generates a SpecifierStr from a string if the string was generated using :meth:`~DMT.core.specifiers.SpecifierStr.string_to_save`."""
         if string.startswith("$"):  # SpecifierStr
             splitted = string[1:].split("#")
@@ -285,6 +311,12 @@ class SpecifierStr(str):
         else:  # it was just a regular str
             return string
 
+    def __add__(
+        self: SpecifierStr, other: Union[SpecifierStr, str, list[Union[str, SpecifierStr]]]
+    ) -> SpecifierStr:
+        """Method stub"""
+        raise NotImplementedError
+
     def __radd__(self, other):
         # if isinstance(other, SpecifierStr): # not needed since __add__ is taking care of this!
         if isinstance(other, str):  # reflected str + SpecifierStr
@@ -294,7 +326,19 @@ class SpecifierStr(str):
         else:
             return NotImplemented
 
-    def __eq__(self, other):
+    def __eq__(self, other: Union[str, SpecifierStr]) -> bool:
+        """Checks if two objects are equal
+
+        Parameters
+        ----------
+        other : str | SpecifierStr
+            Compare to this column name
+
+        Returns
+        -------
+        bool
+            True if equal
+        """
         if isinstance(other, SpecifierStr):
             if self.specifier == other.specifier and self.nodes == other.nodes:
                 # order of subspecifiers does not matter Oo
@@ -313,21 +357,81 @@ class SpecifierStr(str):
                     if self == col_other:
                         return True
         elif other is not None:
-            raise NotImplementedError("Can not compare SpecifierStr to " + type(other))
+            return NotImplemented
 
         return False
 
+    def __contains__(self, other: Union[str, SpecifierStr]) -> bool:
+        """String contains
+
+        Parameters
+        ----------
+        other : str | SpecifierStr
+            Other string or specifier which may contains self
+
+        Returns
+        -------
+        bool
+            str(other) in str(self)
+        """
+        return str(other) in str(self)
+
     def __hash__(self):
         return hash(str(self))
 
+    def get_pint_unit(self) -> pint.Unit:
+        """Declaration of function, to be set later
+
+        Returns
+        -------
+        pint.UnitRegistry
+            [description]
+
+        Raises
+        ------
+        NotImplementedError
+            [description]
+        """
+        raise NotImplementedError()
+
+    def get_descriptor(self) -> str:
+        """Declaration of function, to be set later
+
+        Returns
+        -------
+        str
+            [description]
+
+        Raises
+        ------
+        NotImplementedError
+            [description]
+        """
+        raise NotImplementedError
+
+    def to_tex(self) -> str:
+        """Declaration of function, to be set later
+
+        Returns
+        -------
+        str
+            [description]
+
+        Raises
+        ------
+        NotImplementedError
+            [description]
+        """
+        raise NotImplementedError
+
 
 class GlobalObj(object):
     """A simple class used to store class variables that are global. Also one can iterate over the class attributes in a sorted way."""
 
-    _MEMBERS = None
+    _MEMBERS = []
 
     def __iter__(self):
-        if self._MEMBERS is None:  # only if None or always???
+        if not self._MEMBERS:
             self._set_members()
 
         for elem in list.__iter__(self._MEMBERS):
@@ -349,6 +453,7 @@ class GlobalObj(object):
 # HOLY DEFINITION OF DMT NAMING CONVENTIONS #
 #############################################
 class _sub_specifiers(GlobalObj, metaclass=Singleton):
+
     PERIMETER = SpecifierStr("", sub_specifiers="PERI")
     AREA = SpecifierStr("", sub_specifiers="AREA")
     LENGTH = SpecifierStr("", sub_specifiers="LENGTH")
@@ -459,9 +564,14 @@ class _specifiers_ss_para(GlobalObj, metaclass=Singleton):
     SS_PARA_T = SpecifierStr("T")
 
 
-specifiers_ss_para = _specifiers_ss_para()
-specifiers = _specifiers()
-sub_specifiers = _sub_specifiers()
+specifiers_ss_para: _specifiers_ss_para = _specifiers_ss_para()
+"""Small signal parameters known to DMT."""
+
+specifiers: _specifiers = _specifiers()
+"""Specifiers known to DMT. In a written form these would be the variable."""
+
+sub_specifiers: _sub_specifiers = _sub_specifiers()
+"""Sub specifiers known to DMT. In a written form these would be placed in the subscript of a variable."""
 
 specifiers.add_members(DATA_CONFIG["custom_specifiers"])
 sub_specifiers.add_members(DATA_CONFIG["custom_sub_specifiers"])
@@ -476,7 +586,7 @@ SUB_SPECIFIERS_STR = [
 ##################################################
 
 
-def add(self, other):
+def add(self: SpecifierStr, other: Union[SpecifierStr, str, List[Union[str, SpecifierStr]]]):
     """Method is defined later, since we need the SUB_SPECIFIERS_STR list here...thanks python"""
     if isinstance(other, SpecifierStr):
         spec_new = self.specifier + other.specifier
@@ -503,7 +613,7 @@ def add(self, other):
     elif isinstance(other, list):
         if self.nodes:
             return SpecifierStr(
-                self.specifier, *self.nodes, sub_specifiers=self.sub_specifiers + other
+                self.specifier, *self.nodes, sub_specifiers=self.sub_specifiers + other  # type: ignore
             )
         else:
             return SpecifierStr(self.specifier, *other, sub_specifiers=self.sub_specifiers)
@@ -515,7 +625,7 @@ def add(self, other):
 SpecifierStr.__add__ = add
 
 
-def get_pint_unit(self):
+def get_pint_unit(self) -> pint.Unit:
     """Return the DMT base unit of this specifier as a pint unit"""
     if sub_specifiers.PHASE.sub_specifiers[0] in self.sub_specifiers:
         return unit_registry.degree
@@ -546,7 +656,7 @@ def get_pint_unit(self):
         specifiers.NET_DOPING: 1 / unit_registry.meter / unit_registry.meter / unit_registry.meter,
         specifiers.ACCEPTORS: 1 / unit_registry.meter / unit_registry.meter / unit_registry.meter,
         specifiers.DONNORS: 1 / unit_registry.meter / unit_registry.meter / unit_registry.meter,
-    }
+    }  # type: dict[SpecifierStr, pint.Unit]
     try:
         return unit_converter[self.specifier]
     except KeyError as err:
@@ -586,7 +696,7 @@ def to_tex(self, subscript="", superscript=""):
         Superscript to add to the specifier's Tex representation
 
     Returns
-    ------
+    -------
     tex : str
             A Tex representation fo the specifier.
     """
@@ -693,7 +803,7 @@ def set_col_name(specifier, *nodes, sub_specifiers=None):
     """Returns the valid DMT name for this column.
 
     Parameters
-    -----------
+    ----------
     specifier : str
         Ideally a specifier from :mod:`~DMT.core.specifiers`
     *nodes : str, optional
@@ -701,7 +811,7 @@ def set_col_name(specifier, *nodes, sub_specifiers=None):
     sub_specifiers : str, {'', :py:const:`DMT.core.sub_specifiers.PERIMETER`, :py:const:`DMT.core.sub_specifiers.AREA`}
 
     Returns
-    --------
+    -------
     col : str
         If nodes are given: specifier + '_' + ''.join(nodes)
         Without nodes: specifier
@@ -726,7 +836,7 @@ def get_nodes(string, nodes, fallback=None):
         If value == '', the node is ignored.
 
     Returns
-    ----------
+    -------
     relevant_nodes  :  list of string
         list of strings with the nodes in string.
     """
@@ -882,4 +992,5 @@ natural_scales = {
     specifiers.RESISTANCE: 1,  # Ohm
     specifiers.ENERGY: 1,  # eV
     specifiers.UNILATERAL_GAIN: 1,
+    specifiers.NET_DOPING: 1e-6,  # 1/cm^3
 }
