@@ -1,7 +1,9 @@
 """ DMT supplied build latex command
 """
 # DMT_core
-# Copyright (C) 2019  Markus Müller and Mario Krattenmacher and the DMT contributors <https://gitlab.hrz.tu-chemnitz.de/CEDIC_Bipolar/DMT/>
+# Copyright (C) from 2022  SemiMod
+# Copyright (C) until 2021  Markus Müller, Mario Krattenmacher and Pascal Kuthe
+# <https://gitlab.com/dmt-development/dmt-core>
 #
 # This file is part of DMT_core.
 #
@@ -24,11 +26,11 @@ from pathlib import Path
 from DMT.external.os import cd
 
 
-def build_tex(full_path_to_file, additional_compiler=None, wait=False):
+def build_tex(full_path_to_file, additional_compiler=None, wait=False, extension=None):
     r"""Builds a tex file
 
     Parameters
-    ------------
+    ----------
     full_path_to_file : str or os.Pathlike
     additional_compiler: (str , [str])
         If a additional tex compiler should be used, supply it here. It will be tried as first compiler.
@@ -36,72 +38,69 @@ def build_tex(full_path_to_file, additional_compiler=None, wait=False):
         For example 'latexmk' looks like: ('latexmk', ['--interaction=nonstopmode', '--shell-escape', '--pdf'])
     wait : Boolean, False
         Wait for the build to complete.
+    extension : None, optional
+        If None, assumed extension of Tex file to build is "tex", else the given string is used. Example: extension="tikz"
     """
     if not isinstance(full_path_to_file, Path):
         full_path_to_file = Path(full_path_to_file)
 
+    ext = ".tex"
+    if extension is not None:
+        ext = "." + extension.replace(".", "")
+
     directory = full_path_to_file.parent
     file_name = full_path_to_file.name
-    if file_name.endswith(".tex"):
+    if file_name.endswith(ext):
         file_name = [file_name]
     else:
-        file_name = [file_name + ".tex"]
+        file_name = [file_name + ext]
+
+    compilers = (
+        ("latexmk", ["--interaction=nonstopmode", "--shell-escape", "--pdf"]),
+        ("lualatex", ["--interaction=nonstopmode", "--shell-escape"]),
+        ("pdflatex", ["--interaction=nonstopmode", "--shell-escape"]),
+    )
+    if additional_compiler is not None and additional_compiler[0] is not None:
+        compilers = (additional_compiler,) + compilers
+
+    # turn of testing of all compilers. Just use the first one
+    compiler = compilers[0]
+    command = [compiler[0]] + compiler[1] + file_name
 
     with cd(directory):
-        compilers = (
-            ("latexmk", ["--interaction=nonstopmode", "--shell-escape", "--pdf"]),
-            ("lualatex", ["--interaction=nonstopmode", "--shell-escape"]),
-            ("pdflatex", ["--interaction=nonstopmode", "--shell-escape"]),
-        )
-        if additional_compiler is not None and additional_compiler[0] is not None:
-            compilers = (additional_compiler,) + compilers
-
-        # os_error = None
-
-        for compiler, arguments in compilers:
-            command = [compiler] + arguments + file_name
-
-            # try:
+        try:
             if wait:
                 # output = subprocess.call(command, stderr=subprocess.STDOUT)
                 subprocess.call(command, stderr=subprocess.STDOUT)
             else:
                 # output = subprocess.Popen(command, stderr=subprocess.STDOUT)
                 subprocess.Popen(command, stderr=subprocess.STDOUT)
-            # except (OSError, IOError) as e:
-            #     # Use FileNotFoundError when python 2 is dropped
-            #     os_error = e
+        except FileNotFoundError as err:
+            print(err)
 
-            #     if os_error.errno == errno.ENOENT:
-            #         # If compiler does not exist, try next in the list
-            #         continue
-            #     raise
-            # except subprocess.CalledProcessError as e:
-            #     # For all other errors print the output and raise the error
-            #     print(e.output.decode())
-            #     continue
-            #     # raise
-            # else:
-            #     print(output.decode())
-
-            # Compilation has finished, so no further compilers have to be
-            # tried
-            return
-
-        # Notify user that none of the compilers worked.
-        raise (
-            IOError(
-                "No LaTex compiler was found\n"
-                + "Make sure you have latexmk, lualatex or pdfLaTex installed and in your PATH environment."
+            # Notify user that the compiler is not found.
+            print(
+                "The used latex compiler was not found\n"
+                + "The latex command was "
+                + " ".join(command)
+                + "\n"
+                + "Make sure that a LaTeX distribution (like Tex Live) is installed and in your PATH."
             )
-        )
+        except subprocess.CalledProcessError as err:
+            # For all other errors print the output and raise the error
+            print(err)
+            raise
+        else:
+            print("LaTeX compile successfull")
+
+        # Compilation has finished
 
 
 def build_png(full_path_to_file, wait=True):
     r"""Builds a tex file to png graphic.
 
     Parameters
-    ------------
+    ----------
     full_path_to_file : str or os.Pathlike
 
     TODO
@@ -144,7 +143,7 @@ def build_svg(full_path_to_file, wait=True):
     r"""Builds a tex file to scalable vector graphic.
 
     Parameters
-    ------------
+    ----------
     full_path_to_file : str or os.Pathlike
     """
     if not isinstance(full_path_to_file, Path):
@@ -180,7 +179,7 @@ def clean_tex_files(directory, file_name, keep=(".tex", ".pdf"), regex=False):
     """Deletes all intermediate files in the directory with name file_name except the tex and pdf.
 
     Parameters
-    ---------------
+    ----------
     directory : str
         Direcory to clean
     file_name : str
@@ -203,3 +202,86 @@ def clean_tex_files(directory, file_name, keep=(".tex", ".pdf"), regex=False):
     for file_curr in Path(directory).iterdir():
         if test_fct(file_name, file_curr) and (file_curr.suffix not in keep):
             file_curr.unlink()
+
+
+SI_UNITS_CONVERTER = {
+    r"\second": "s",
+    r"\hertz": "Hz",
+    r"\volt": "V",
+    r"\watt": "W",
+    r"\ampere": "A",
+    r"\kelvin": "K",
+    r"\ohm": "Ohm",
+    r"\siemens": "S",
+    r"\farad": "F",
+    r"\coulomb": "C",
+    r"\electronvolt": "eV",
+    r"\meter": "m",
+    r"\metre": "m",
+    r"\per": "inv",
+    r"\square": "sq",
+    r"\cubic": "cu",
+    r"\giga": "G",
+    r"\centi": "c",
+    r"\milli": "m",
+    r"\micro": "u",
+    r"\nano": "n",
+    r"\femto": "f",
+    r"\pico": "p",
+}
+
+
+def resolve_siunitx(label):
+    """This function tries to remove siunitx expressions from given Tex expression."""
+    regex_si = r"\\si({.*?})"
+    regex_SI = r"\\SI({.*?})({.*?})"
+    if "si" in label:
+        pattern = re.compile(regex_si)
+        for match in pattern.findall(label):
+            units = match[1:-1]
+            # todo: resolve unit after unit, then for each unit the scaler (milli and so on), than stuff like per. Only then per can be correctly replaced.
+            for key in SI_UNITS_CONVERTER:
+                units = units.replace(key, SI_UNITS_CONVERTER[key])
+
+            label = label.replace(match, units)
+
+    if "SI" in label:
+        pattern = re.compile(regex_SI)
+        for match_number, match_unit in pattern.findall(label):
+            # match 0 is number
+            number = match_number[1:-1]
+            label = label.replace(match_number, number)
+
+            # match 1 is unit
+            units = match_unit[1:-1]
+            for key in SI_UNITS_CONVERTER:
+                units = units.replace(key, SI_UNITS_CONVERTER[key])
+
+            label = label.replace(match_unit, units)
+
+    if "underline" in label:
+        regex_underline = r"(\\underline{)([^{}]+)(})"
+        pattern = re.compile(regex_underline)
+        for match in pattern.findall(label):
+            label = label.replace(match[0], "")
+            label = label.replace(match[1] + match[2], match[1])
+
+    # drop some tex things that are not understood by pandoc
+    label = label.replace("\\si", "")
+    label = label.replace("\\SI", "")
+    label = label.replace("\\,", " ")
+
+    return label
+
+
+def tex_to_text(tex):
+    """Return a text representation of a tex label."""
+    tex = resolve_siunitx(tex)
+    tex = tex.replace("\\num", "")
+    tex = tex.replace("\\mathrm", "")
+    tex = tex.replace("\\left(", "(")
+    tex = tex.replace("\\right)", ")")
+    tex = tex.replace("{", "")
+    tex = tex.replace("}", "")
+    tex = tex.replace("$", "")
+    return tex
